1#ifndef AMREX_MLALAPLACIAN_H_
2#define AMREX_MLALAPLACIAN_H_
3#include <AMReX_Config.H>
6#include <AMReX_MLALap_K.H>
25 using FAB =
typename MF::fab_type;
26 using RT =
typename MF::value_type;
81 [[nodiscard]]
int getNComp ()
const override {
return m_ncomp; }
91 [[nodiscard]]
bool isSingular (
int amrlev) const final {
return m_is_singular[amrlev]; }
95 void Fapply (
int amrlev,
int mglev, MF& out,
const MF& in)
const final;
97 void Fsmooth (
int amrlev,
int mglev, MF& sol,
const MF& rhs,
int redblack)
const final;
106 int face_only=0) const final;
109 void normalize (
int amrlev,
int mglev, MF& mf) const final;
116 [[nodiscard]] MF
const*
getACoeffs (
int amrlev,
int mglev)
const final
117 {
return &(m_a_coeffs[amrlev][mglev]); }
122 [[nodiscard]] std::unique_ptr<MLLinOpT<MF>>
makeNLinOp (
int )
const final {
123 amrex::Abort(
"MLALaplacian::makeNLinOp: Not implemented");
124 return std::unique_ptr<MLLinOpT<MF>>{};
145 bool m_needs_update =
true;
147 RT m_a_scalar = std::numeric_limits<RT>::quiet_NaN();
148 RT m_b_scalar = std::numeric_limits<RT>::quiet_NaN();
155 void updateSingularFlag ();
158template <
typename MF>
167 define(a_geom, a_grids, a_dmap, a_info, a_factory);
170template <
typename MF>
182 const int ncomp = this->getNComp();
184 m_a_coeffs.resize(this->m_num_amr_levels);
185 for (
int amrlev = 0; amrlev < this->m_num_amr_levels; ++amrlev)
187 m_a_coeffs[amrlev].resize(this->m_num_mg_levels[amrlev]);
188 for (
int mglev = 0; mglev < this->m_num_mg_levels[amrlev]; ++mglev)
190 m_a_coeffs[amrlev][mglev].define(this->m_grids[amrlev][mglev],
191 this->m_dmap[amrlev][mglev], ncomp, 0);
196template <
typename MF>
199template <
typename MF>
207 for (
int amrlev = 0; amrlev < this->m_num_amr_levels; ++amrlev)
209 m_a_coeffs[amrlev][0].setVal(
RT(0.0));
214template <
typename MF>
218 const int ncomp = this->getNComp();
219 m_a_coeffs[amrlev][0].LocalCopy(alpha, 0, 0, ncomp,
IntVect(0));
220 m_needs_update =
true;
223template <
typename MF>
227 BL_PROFILE(
"MLALaplacian::averageDownCoeffs()");
229 for (
int amrlev = this->m_num_amr_levels-1; amrlev > 0; --amrlev)
231 auto& fine_a_coeffs = m_a_coeffs[amrlev];
233 averageDownCoeffsSameAmrLevel(amrlev, fine_a_coeffs);
234 averageDownCoeffsToCoarseAmrLevel(amrlev);
237 averageDownCoeffsSameAmrLevel(0, m_a_coeffs[0]);
240template <
typename MF>
244 const int ncomp = this->getNComp();
245 const int nmglevs = a.
size();
246 for (
int mglev = 1; mglev < nmglevs; ++mglev)
248 if (m_a_scalar ==
RT(0.0))
250 a[mglev].setVal(
RT(0.0));
254 AMREX_ASSERT(amrlev == 0 || !this->hasHiddenDimension());
255 IntVect ratio = (amrlev > 0) ?
IntVect(this->mg_coarsen_ratio) : this->mg_coarsen_ratio_vec[mglev-1];
261template <
typename MF>
265 const int ncomp = this->getNComp();
266 auto& fine_a_coeffs = m_a_coeffs[flev ].back();
267 auto& crse_a_coeffs = m_a_coeffs[flev-1].front();
269 if (m_a_scalar !=
RT(0.0)) {
276template <
typename MF>
280 m_is_singular.clear();
281 m_is_singular.resize(this->m_num_amr_levels,
false);
282 auto itlo = std::ranges::find(this->m_lobc[0], BCType::Dirichlet);
283 auto ithi = std::ranges::find(this->m_hibc[0], BCType::Dirichlet);
284 if (itlo == this->m_lobc[0].
end() && ithi == this->m_hibc[0].
end())
286 for (
int alev = 0; alev < this->m_num_amr_levels; ++alev)
288 if (this->m_domain_covered[alev])
290 if (m_a_scalar == RT(0.0))
292 m_is_singular[alev] =
true;
298 RT asum = m_a_coeffs[alev].back().sum(0,
IntVect(0));
299 RT amax = m_a_coeffs[alev].back().norminf(0,1,
IntVect(0));
300 m_is_singular[alev] = (asum <= amax * RT(1.e-12));
307template <
typename MF>
311 BL_PROFILE(
"MLALaplacian::prepareForSolve()");
314 updateSingularFlag();
315 m_needs_update =
false;
318template <
typename MF>
324 updateSingularFlag();
325 m_needs_update =
false;
328template <
typename MF>
334 const int ncomp = this->getNComp();
336 const MF& acoef = m_a_coeffs[amrlev][mglev];
340 RT(this->m_geom[amrlev][mglev].InvCellSize(1)),
341 RT(this->m_geom[amrlev][mglev].InvCellSize(2)))};
342#if (AMREX_SPACEDIM < 3)
343 const RT dx =
RT(this->m_geom[amrlev][mglev].CellSize(0));
344 const RT probxlo =
RT(this->m_geom[amrlev][mglev].ProbLo(0));
347#if (AMREX_SPACEDIM == 3)
349 this->get_d1(dxinv[0], dxinv[1], dxinv[2])};
352 const RT ascalar = m_a_scalar;
353 const RT bscalar = m_b_scalar;
356#pragma omp parallel if (Gpu::notInLaunchRegion())
360 const Box& bx = mfi.tilebox();
361 const auto& xfab = in.array(mfi);
362 const auto& yfab = out.array(mfi);
363 const auto& afab = acoef.array(mfi);
365#if (AMREX_SPACEDIM != 3)
366 if (this->m_has_metric_term) {
369 mlalap_adotx_m(tbx, yfab, xfab, afab, dxinv, ascalar, bscalar, dx, probxlo, ncomp);
374 mlalap_adotx(tbx, yfab, xfab, afab, dxinv, ascalar, bscalar, ncomp);
378 if (this->hasHiddenDimension()) {
379 Box const& bx2d = this->compactify(bx);
380 const auto& xfab2d = this->compactify(xfab);
381 const auto& yfab2d = this->compactify(yfab);
382 const auto& afab2d = this->compactify(afab);
385 TwoD::mlalap_adotx(tbx2d, yfab2d, xfab2d, afab2d, dhinv, ascalar, bscalar, ncomp);
390 mlalap_adotx(tbx, yfab, xfab, afab, dxinv, ascalar, bscalar, ncomp);
397template <
typename MF>
403 const int ncomp = this->getNComp();
405 const MF& acoef = m_a_coeffs[amrlev][mglev];
409 RT(this->m_geom[amrlev][mglev].InvCellSize(1)),
410 RT(this->m_geom[amrlev][mglev].InvCellSize(2)))};
411#if (AMREX_SPACEDIM < 3)
412 const RT dx =
RT(this->m_geom[amrlev][mglev].CellSize(0));
413 const RT probxlo =
RT(this->m_geom[amrlev][mglev].ProbLo(0));
416#if (AMREX_SPACEDIM == 3)
418 this->get_d1(dxinv[0], dxinv[1], dxinv[2])};
421 const RT ascalar = m_a_scalar;
422 const RT bscalar = m_b_scalar;
425#pragma omp parallel if (Gpu::notInLaunchRegion())
429 const Box& bx = mfi.tilebox();
430 const auto& fab = mf.array(mfi);
431 const auto& afab = acoef.array(mfi);
433#if (AMREX_SPACEDIM != 3)
434 if (this->m_has_metric_term) {
437 mlalap_normalize_m(tbx, fab, afab, dxinv, ascalar, bscalar, dx, probxlo, ncomp);
442 mlalap_normalize(tbx, fab, afab, dxinv, ascalar, bscalar, ncomp);
446 if (this->hasHiddenDimension()) {
447 Box const& bx2d = this->compactify(bx);
448 const auto& fab2d = this->compactify(fab);
449 const auto& afab2d = this->compactify(afab);
452 TwoD::mlalap_normalize(tbx2d, fab2d, afab2d, dhinv, ascalar, bscalar, ncomp);
457 mlalap_normalize(tbx, fab, afab, dxinv, ascalar, bscalar, ncomp);
464template <
typename MF>
470 const int ncomp = this->getNComp();
472 const MF& acoef = m_a_coeffs[amrlev][mglev];
473 const auto& undrrelxr = this->m_undrrelxr[amrlev][mglev];
474 const auto& maskvals = this->m_maskvals [amrlev][mglev];
478 const auto& f0 = undrrelxr[oitr()]; ++oitr;
479 const auto& f1 = undrrelxr[oitr()]; ++oitr;
480#if (AMREX_SPACEDIM > 1)
481 const auto& f2 = undrrelxr[oitr()]; ++oitr;
482 const auto& f3 = undrrelxr[oitr()]; ++oitr;
483#if (AMREX_SPACEDIM > 2)
484 const auto& f4 = undrrelxr[oitr()]; ++oitr;
485 const auto& f5 = undrrelxr[oitr()]; ++oitr;
491#if (AMREX_SPACEDIM > 1)
494#if (AMREX_SPACEDIM > 2)
500 const Real* dxinv = this->m_geom[amrlev][mglev].InvCellSize();
502 const RT dhy = m_b_scalar*
RT(dxinv[1]*dxinv[1]);,
503 const RT dhz = m_b_scalar*
RT(dxinv[2]*dxinv[2]););
505#if (AMREX_SPACEDIM == 3)
506 RT dh0 = this->get_d0(dhx, dhy, dhz);
507 RT dh1 = this->get_d1(dhx, dhy, dhz);
510#if (AMREX_SPACEDIM < 3)
511 const RT dx =
RT(this->m_geom[amrlev][mglev].CellSize(0));
512 const RT probxlo =
RT(this->m_geom[amrlev][mglev].ProbLo(0));
515 const RT alpha = m_a_scalar;
521#pragma omp parallel if (Gpu::notInLaunchRegion())
525 const auto& m0 = mm0.
array(mfi);
526 const auto& m1 = mm1.
array(mfi);
527#if (AMREX_SPACEDIM > 1)
528 const auto& m2 = mm2.
array(mfi);
529 const auto& m3 = mm3.
array(mfi);
530#if (AMREX_SPACEDIM > 2)
531 const auto& m4 = mm4.
array(mfi);
532 const auto& m5 = mm5.
array(mfi);
536 const Box& tbx = mfi.tilebox();
537 const Box& vbx = mfi.validbox();
538 const auto& solnfab = sol.array(mfi);
539 const auto& rhsfab = rhs.array(mfi);
540 const auto& afab = acoef.array(mfi);
542 const auto& f0fab = f0.array(mfi);
543 const auto& f1fab = f1.array(mfi);
544#if (AMREX_SPACEDIM > 1)
545 const auto& f2fab = f2.array(mfi);
546 const auto& f3fab = f3.array(mfi);
547#if (AMREX_SPACEDIM > 2)
548 const auto& f4fab = f4.array(mfi);
549 const auto& f5fab = f5.array(mfi);
553#if (AMREX_SPACEDIM == 1)
554 if (this->m_has_metric_term) {
557 mlalap_gsrb_m(thread_box, solnfab, rhsfab, alpha, dhx,
567 mlalap_gsrb(thread_box, solnfab, rhsfab, alpha, dhx,
571 vbx, redblack, ncomp);
577#if (AMREX_SPACEDIM == 2)
578 if (this->m_has_metric_term) {
581 mlalap_gsrb_m(thread_box, solnfab, rhsfab, alpha, dhx, dhy,
593 mlalap_gsrb(thread_box, solnfab, rhsfab, alpha, dhx, dhy,
599 vbx, redblack, ncomp);
604#if (AMREX_SPACEDIM == 3)
605 if (this->hasHiddenDimension()) {
606 Box const& tbx_2d = this->compactify(tbx);
607 Box const& vbx_2d = this->compactify(vbx);
608 const auto& solnfab_2d = this->compactify(solnfab);
609 const auto& rhsfab_2d = this->compactify(rhsfab);
610 const auto& afab_2d = this->compactify(afab);
611 const auto& f0fab_2d = this->compactify(this->get_d0(f0fab,f1fab,f2fab));
612 const auto& f1fab_2d = this->compactify(this->get_d1(f0fab,f1fab,f2fab));
613 const auto& f2fab_2d = this->compactify(this->get_d0(f3fab,f4fab,f5fab));
614 const auto& f3fab_2d = this->compactify(this->get_d1(f3fab,f4fab,f5fab));
615 const auto& m0_2d = this->compactify(this->get_d0(m0,m1,m2));
616 const auto& m1_2d = this->compactify(this->get_d1(m0,m1,m2));
617 const auto& m2_2d = this->compactify(this->get_d0(m3,m4,m5));
618 const auto& m3_2d = this->compactify(this->get_d1(m3,m4,m5));
621 TwoD::mlalap_gsrb(thread_box, solnfab_2d, rhsfab_2d, alpha, dh0, dh1,
627 vbx_2d, redblack, ncomp);
632 mlalap_gsrb(thread_box, solnfab, rhsfab, alpha, dhx, dhy, dhz,
640 vbx, redblack, ncomp);
647template <
typename MF>
655 const int ncomp = this->getNComp();
658 const Real* dxinv = this->m_geom[amrlev][mglev].InvCellSize();
661 const auto& fyarr = flux[1]->array();,
662 const auto& fzarr = flux[2]->array(););
663 const auto& solarr = sol.array();
665#if (AMREX_SPACEDIM != 3)
666 const RT dx =
RT(this->m_geom[amrlev][mglev].CellSize(0));
667 const RT probxlo =
RT(this->m_geom[amrlev][mglev].ProbLo(0));
670#if (AMREX_SPACEDIM == 3)
672 if (this->hiddenDirection() != 0) {
673 RT fac = m_b_scalar *
RT(dxinv[0]);
678 mlalap_flux_xface(tbox, fxarr, solarr, fac, blen, ncomp);
681 flux[0]->template setVal<RunOn::Device>(
RT(0.0));
683 if (this->hiddenDirection() != 1) {
684 RT fac = m_b_scalar *
RT(dxinv[1]);
689 mlalap_flux_yface(tbox, fyarr, solarr, fac, blen, ncomp);
692 flux[1]->template setVal<RunOn::Device>(
RT(0.0));
694 if (this->hiddenDirection() != 2) {
695 RT fac = m_b_scalar *
RT(dxinv[2]);
700 mlalap_flux_zface(tbox, fzarr, solarr, fac, blen, ncomp);
703 flux[2]->template setVal<RunOn::Device>(
RT(0.0));
706 if (this->hiddenDirection() != 0) {
707 RT fac = m_b_scalar *
RT(dxinv[0]);
711 mlalap_flux_x(tbox, fxarr, solarr, fac, ncomp);
714 flux[0]->template setVal<RunOn::Device>(
RT(0.0));
716 if (this->hiddenDirection() != 1) {
717 RT fac = m_b_scalar *
RT(dxinv[1]);
721 mlalap_flux_y(tbox, fyarr, solarr, fac, ncomp);
724 flux[1]->template setVal<RunOn::Device>(
RT(0.0));
726 if (this->hiddenDirection() != 2) {
727 RT fac = m_b_scalar *
RT(dxinv[2]);
731 mlalap_flux_z(tbox, fzarr, solarr, fac, ncomp);
734 flux[2]->template setVal<RunOn::Device>(
RT(0.0));
737#elif (AMREX_SPACEDIM == 2)
739 if (this->hiddenDirection() != 0) {
740 RT fac = m_b_scalar *
RT(dxinv[0]);
743 if (this->m_has_metric_term) {
746 mlalap_flux_xface_m(tbox, fxarr, solarr, fac, blen, dx, probxlo, ncomp);
751 mlalap_flux_xface(tbox, fxarr, solarr, fac, blen, ncomp);
755 flux[0]->template setVal<RunOn::Device>(
RT(0.0));
757 if (this->hiddenDirection() != 1) {
758 RT fac = m_b_scalar *
RT(dxinv[1]);
761 if (this->m_has_metric_term) {
764 mlalap_flux_yface_m(tbox, fyarr, solarr, fac, blen, dx, probxlo, ncomp);
769 mlalap_flux_yface(tbox, fyarr, solarr, fac, blen, ncomp);
773 flux[1]->template setVal<RunOn::Device>(
RT(0.0));
776 if (this->hiddenDirection() != 0) {
777 RT fac = m_b_scalar *
RT(dxinv[0]);
779 if (this->m_has_metric_term) {
782 mlalap_flux_x_m(tbox, fxarr, solarr, fac, dx, probxlo, ncomp);
787 mlalap_flux_x(tbox, fxarr, solarr, fac, ncomp);
791 flux[0]->template setVal<RunOn::Device>(
RT(0.0));
793 if (this->hiddenDirection() != 1) {
794 RT fac = m_b_scalar *
RT(dxinv[1]);
796 if (this->m_has_metric_term) {
799 mlalap_flux_y_m(tbox, fyarr, solarr, fac, dx, probxlo, ncomp);
804 mlalap_flux_y(tbox, fyarr, solarr, fac, ncomp);
808 flux[1]->template setVal<RunOn::Device>(
RT(0.0));
813 RT fac = m_b_scalar *
RT(dxinv[0]);
816 if (this->m_has_metric_term) {
819 mlalap_flux_xface_m(tbox, fxarr, solarr, fac, blen, dx, probxlo, ncomp);
824 mlalap_flux_xface(tbox, fxarr, solarr, fac, blen, ncomp);
828 RT fac = m_b_scalar *
RT(dxinv[0]);
830 if (this->m_has_metric_term) {
833 mlalap_flux_x_m(tbox, fxarr, solarr, fac, dx, probxlo, ncomp);
838 mlalap_flux_x(tbox, fxarr, solarr, fac, ncomp);
#define BL_PROFILE(a)
Definition AMReX_BLProfiler.H:551
#define AMREX_ASSERT(EX)
Definition AMReX_BLassert.H:38
#define AMREX_GPU_LAUNCH_HOST_DEVICE_LAMBDA_RANGE(TN, TI, block)
Definition AMReX_GpuLaunchMacrosC.nolint.H:4
#define AMREX_D_TERM(a, b, c)
Definition AMReX_SPACE.H:172
#define AMREX_D_DECL(a, b, c)
Definition AMReX_SPACE.H:171
__host__ __device__ IntVectND< dim > length() const noexcept
Return the length of the BoxND.
Definition AMReX_Box.H:167
Definition AMReX_FabFactory.H:50
Iterator for looping ever tiles and boxes of amrex::FabArray based containers.
Definition AMReX_MFIter.H:88
Box tilebox() const noexcept
Return the tile Box at the current index.
Definition AMReX_MFIter.cpp:389
bool isValid() const noexcept
Is the iterator valid i.e. is it associated with a FAB?
Definition AMReX_MFIter.H:172
Multi-component ALaplacian (a scalar plus optional spatial a coeffs).
Definition AMReX_MLALaplacian.H:22
RT getAScalar() const final
Scalar alpha applied to the a term.
Definition AMReX_MLALaplacian.H:112
MLALaplacianT< MF > & operator=(const MLALaplacianT< MF > &)=delete
MLALaplacianT(const MLALaplacianT< MF > &)=delete
void averageDownCoeffsToCoarseAmrLevel(int flev)
Average a coefficients from fine AMR level flev to flev-1.
Definition AMReX_MLALaplacian.H:263
~MLALaplacianT() override
typename MF::fab_type FAB
Definition AMReX_MLALaplacian.H:25
void setScalars(RT a, RT b) noexcept
Set constant scalars a and b in a \phi - b \nabla^2 \phi.
Definition AMReX_MLALaplacian.H:201
void FFlux(int amrlev, const MFIter &mfi, const Array< FAB *, 3 > &flux, const FAB &sol, Location, int face_only=0) const final
Produce face fluxes on AMR level amrlev for the tilebox described by mfi using sol,...
Definition AMReX_MLALaplacian.H:649
void update() override
Update for reuse.
Definition AMReX_MLALaplacian.H:320
bool isBottomSingular() const final
Shortcut for the coarsest level singular flag.
Definition AMReX_MLALaplacian.H:93
std::unique_ptr< MLLinOpT< MF > > makeNLinOp(int) const final
Create the NSolve counterpart of this operator with the requested grid size.
Definition AMReX_MLALaplacian.H:122
Array< MF const *, 3 > getBCoeffs(int, int) const final
ALaplacian has no b coefficients; this returns null pointers.
Definition AMReX_MLALaplacian.H:119
MF const * getACoeffs(int amrlev, int mglev) const final
Access the stored a coefficient MultiFab for (amrlev,mglev).
Definition AMReX_MLALaplacian.H:116
void prepareForSolve() final
Complete per-level setup (averaging, singularity flags) before solving.
Definition AMReX_MLALaplacian.H:309
void Fapply(int amrlev, int mglev, MF &out, const MF &in) const final
Apply the ALaplacian to in (writing out) on (amrlev,mglev).
Definition AMReX_MLALaplacian.H:330
void averageDownCoeffs()
Average a coefficients down across all AMR and MG levels.
Definition AMReX_MLALaplacian.H:225
MLALaplacianT(MLALaplacianT< MF > &&)=delete
bool isSingular(int amrlev) const final
True if level amrlev is singular.
Definition AMReX_MLALaplacian.H:91
MLALaplacianT()=default
Construct an empty operator; call define() before use.
void normalize(int amrlev, int mglev, MF &mf) const final
Divide mf by the diagonal of the operator (used by CG-family bottom solvers).
Definition AMReX_MLALaplacian.H:399
void Fsmooth(int amrlev, int mglev, MF &sol, const MF &rhs, int redblack) const final
Run a smoothing sweep on (amrlev,mglev). redblack selects the red (0) or black (1) half of the grid.
Definition AMReX_MLALaplacian.H:466
bool needsUpdate() const override
Does it need update if it's reused?
Definition AMReX_MLALaplacian.H:83
void define(const Vector< Geometry > &a_geom, const Vector< BoxArray > &a_grids, const Vector< DistributionMapping > &a_dmap, const LPInfo &a_info=LPInfo(), const Vector< FabFactory< FAB > const * > &a_factory={})
Bind the operator to an AMR hierarchy (no overset support).
Definition AMReX_MLALaplacian.H:172
RT getBScalar() const final
Scalar beta applied to the Laplacian term.
Definition AMReX_MLALaplacian.H:114
void averageDownCoeffsSameAmrLevel(int amrlev, Vector< MF > &a)
Average a coefficients down within a single AMR level (fine-to-coarse MG).
Definition AMReX_MLALaplacian.H:242
typename MF::value_type RT
Definition AMReX_MLALaplacian.H:26
int getNComp() const override
Return number of components.
Definition AMReX_MLALaplacian.H:81
void setACoeffs(int amrlev, const MF &alpha)
Provide per-cell a coefficients on AMR level amrlev (stored directly in alpha).
Definition AMReX_MLALaplacian.H:216
typename MLLinOpT< MF >::Location Location
Definition AMReX_MLALaplacian.H:29
Cell-centered operator that exposes ABec Laplacian helpers to derived classes.
Definition AMReX_MLCellABecLap.H:22
void define(const Vector< Geometry > &a_geom, const Vector< BoxArray > &a_grids, const Vector< DistributionMapping > &a_dmap, const LPInfo &a_info=LPInfo(), const Vector< FabFactory< FAB > const * > &a_factory={})
Describe the AMR hierarchy when overset masks are not required.
Definition AMReX_MLCellABecLap.H:150
void prepareForSolve() override
Standard hook called before MLMG iterates (fixes BC data, etc.).
Definition AMReX_MLCellABecLap.H:305
void update() override
Average coefficients/metrics when marked dirty.
Definition AMReX_MLCellABecLap.H:298
Definition AMReX_MultiMask.H:23
Array4< int const > array(const MFIter &mfi) const noexcept
Return an Array4 view (const) for iterator mfi.
Definition AMReX_MultiMask.H:69
An Iterator over the Orientation of Faces of a Box.
Definition AMReX_Orientation.H:135
This class is a thin wrapper around std::vector. Unlike vector, Vector::operator[] provides bound che...
Definition AMReX_Vector.H:29
Long size() const noexcept
Definition AMReX_Vector.H:54
amrex_real Real
Floating Point Type for Fields.
Definition AMReX_REAL.H:79
__host__ __device__ BoxND< dim > surroundingNodes(const BoxND< dim > &b, int dir) noexcept
Return a BoxND with NODE based coordinates in direction dir that encloses BoxND b.
Definition AMReX_Box.H:1582
__host__ __device__ BoxND< dim > bdryLo(const BoxND< dim > &b, int dir, int len=1) noexcept
Return the BoxND of length len on the low boundary of b along coordinate direction dir.
Definition AMReX_Box.H:1715
std::array< T, N > Array
Definition AMReX_Array.H:31
bool notInLaunchRegion() noexcept
Definition AMReX_GpuControl.H:89
Definition AMReX_Amr.cpp:50
void average_down(const MultiFab &S_fine, MultiFab &S_crse, const Geometry &fgeom, const Geometry &cgeom, int scomp, int ncomp, int rr)
Definition AMReX_MultiFabUtil.cpp:359
LinOpBCType
Definition AMReX_LO_BCTYPES.H:27
IntVectND< 3 > IntVect
IntVect is an alias for amrex::IntVectND instantiated with AMREX_SPACEDIM.
Definition AMReX_BaseFwd.H:38
bool TilingIfNotGPU() noexcept
Definition AMReX_MFIter.H:12
void Abort(const std::string &msg)
Print a fatal-error message to stderr and abort execution.
Definition AMReX.cpp:241
__host__ __device__ Dim3 end(BoxND< dim > const &box) noexcept
Return the iterator end coordinate of box as Dim3.
Definition AMReX_Box.H:2257
Fixed-size array that can be used on GPU.
Definition AMReX_Array.H:52
Configuration knobs for multilevel linear operators (grid agglomeration, metrics, etc....
Definition AMReX_MLLinOp.H:51
Location
Definition AMReX_MLLinOp.H:119
Definition AMReX_MFIter.H:20
MFItInfo & SetDynamic(bool f) noexcept
Definition AMReX_MFIter.H:43
MFItInfo & EnableTiling(const IntVect &ts=FabArrayBase::mfiter_tile_size) noexcept
Definition AMReX_MFIter.H:31